
{% extends "test_base.html" %} 

{% block main_header %}Test Groups{% endblock %}


{% block body_content %}
        <ul>
            {% for key,route in routes.items() %}
                <li>
                    <a href="{{key}}">{{key}} => {{route.functionName}}</a>
                </li>
            {% endfor %}
        </ul>
{% endblock %}


{% block page_script %}
  <script>
    
    document.addEventListener('DOMContentLoaded', function () {
       
    });

  </script>
  <script type="module" src="/webui/static/zwc/wc_form_input.js"></script> 
  <script type="module" src="/webui/static/zwc/wc_tab.js"></script> 
  <script type="module" src="/webui/static/zwc/wc_form_group.js"></script> 
  <script type="module" src="/webui/static/zwc/wc_table.js"></script> 
  <script type="module" src="/webui/static/zwc/wc_form_checkbox.js"></script> 
  <script type="module" src="/webui/static/zwc/wc_modal.js"></script> 
  <script type="module" src="/webui/static/zwc/wc_form_radio.js"></script> 


{% endblock %}