
{% extends "zbase/base.html" %}
 	

{% block main_body %}
	<!-- {{ self._TemplateReference__context.name }} START -->
 
 	<section class=" ">
	{% block body_content %}{% endblock %}
	
        <!-- Footer-->
        <footer class="footer py-4">
            <div class="container">
            	<div class="row align-items-center">
            		 <hr class="  " style="width: 90%;">  
            	</div>
                <div class="row align-items-start ">
                    <div class="col-lg-3 text-lg-left">
                    	<a href="{{ home_url }}" class="b-brand">
                      		<img src="/static/images/logo-dark.svg" alt="" class="logo logo-lg">
                  		</a>
                	</div> 
                    <div class="col-lg-3 text-lg-left  ">
                    	<h5 class="font-inverse">Navigation</h5>
                    	<nav>
                    		<div><a href="{{ url_for('homepage.home')}}">Home</a></div>
                    		<div><a href="{{ url_for('homepage.home')}}">Blog</a></div>
                    		<div><a class=" " href="{{ url_for('homepage.aboutus')}}">About Us</a> </div>
                        	 
                    	</nav>
                    </div> 
                    <div class="col-lg-3 text-lg-left "  >
                    	<h5 class="font-inverse">Resources</h5>
                    	<nav> 
                			 <div><a class=" " href="{{url_for('homepage.faq')}}">FAQ</a> </div>
                			 <div><a class=" " href="#!">Privacy Policy</a> </div>
	                         <div><a href="#!">Terms of Use</a> </div>
	                         <div><a href="#!">Contact Us</a> </div>
	                	</nav>	
                    </div>                    

                    <div class="col-lg-3 text-lg-left  ">
                        <h5 class="font-inverse">Social Media</h5>
                        <nav>
                        	<div><a class=" " href="#!">Twitter <i class="fab fa-twitter"></i> </a></div>
	                        <div><a class=" " href="#!">Facebook <i class="fab fa-facebook-f"></i></a></div>
	                        <div><a class=" " href="#!">LinkedIn <i class="fab fa-linkedin-in"></i></a></div>
                        </nav>  
                    </div>
                </div>
            </div>
        </footer>
 
	</section>


	<!-- {{ self._TemplateReference__context.name }} END -->
{% endblock %}




  
