{% extends "zerrors/base_error.html" %}
 
{% block body_content %}
	<!-- user_noaccess: body_content start -->

			<div class="row justify-content-center">
			    <div class="col-md-8">
	                <div class="text-center">
	                    <div class="img"><img src="config.CONTENT.ERROR_IMAGE_NO_ACCESS" alt="" class="img-fluid" ></div>
	                    <h5 class="text-muted my-4">Sorry - no access</h5>
	                    <form action="{{ url_for('main') }}">
	                        <button class="btn waves-effect waves-light btn-primary mb-4"><i class="feather icon-refresh-ccw mr-2"></i>Back to Home</button>
	                    </form>
	                </div>
            	</div>
			</div>
 
    <!-- user_noaccess: body_content end -->
   
{% endblock %}