
{% extends "test_base.html" %} 

{% block main_header %}Test Groups{% endblock %}


{% block body_content %}
        
<wc-button id="i_btn_01" action="/button/click" label="generate"></wc-button>

{% endblock %}


{% block page_script %}

  <script>
    
    document.addEventListener('DOMContentLoaded', function () {
       
    });

  </script>
 
 
    <script type="module" src="/webui/static/zwc/wc_all.js"></script> 
 

{% endblock %}