<html>
    <head>
        <link rel="stylesheet" href="/webui/static/zcss/main.css" crossorigin="anonymous" /> 
    </head>
    <body>
        
        <h1>{% block main_header %}{% endblock %}</h1>

        {% block body_content %}

        {% endblock %}

        {% block page_script %}
        {% endblock %}

    </body>

    
</html>