"""empty message

Revision ID: 3c00080e3336
Revises: 
Create Date: 2022-04-18 01:54:36.181293

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '3c00080e3336'
down_revision = None
branch_labels = None
depends_on = None


def upgrade():
    pass


def downgrade():
    pass
