import WCInput from  '/webui/static/zwc/wc_form_input.js' ;
import WCTab from  '/webui/static/zwc/wc_tab.js' ;

import WCTable from  '/webui/static/zwc/wc_table.js' ;
import WCCheckbox from  '/webui/static/zwc/wc_form_checkbox.js' ;
import WCModal from  '/webui/static/zwc/wc_modal.js' ;
import WCRadio from  '/webui/static/zwc/wc_form_radio.js' ;
import WCNav from  '/webui/static/zwc/wc_nav.js' ;

// import WCPanel from  '/webui/static/zwc/wc_panel.js' ;

import WCGroup from  '/webui/static/zwc/wc_form_group.js' ;


// <!-- <script  src="/common/st/_def/common/static/_def/js/common_utils.js"></script>  -->
//   <script type="module" src="/webui/static/zwc/wc_form_input.js"></script> 
//   <!-- <script type="module" src="/common/st/_def/commonui/static/_def/wc/wc_form_checkbox.js"></script>  -->
//   <script type="module" src="/webui/static/zwc/wc_table.js"></script> 
//   <script type="module" src="/webui/static/zwc/wc_modal.js"></script> 
//   <script type="module" src="/webui/static/zwc/wc_tab.js"></script> 
//   <!-- <script type="module" src="/webui/static/zwc/wc_panel.js"></script>  -->
//   <script type="module" src="/webui/static/zwc/wc_form_checkbox.js"></script> 


// import WCTab from  '/webui/static/zwc/wc_form_radio.js' ;


// <script type="module" src="/webui/static/zwc/wc_form_input.js"></script> 
//   <script type="module" src="/webui/static/zwc/wc_tab.js"></script> 
  
//   <script type="module" src="/webui/static/zwc/wc_table.js"></script> 
//   <script type="module" src="/webui/static/zwc/wc_form_checkbox.js"></script> 
//   <script type="module" src="/webui/static/zwc/wc_modal.js"></script> 
//   <script type="module" src="/webui/static/zwc/wc_form_radio.js"></script> 

//   <script type="module" src="/webui/static/zwc/wc_form_group.js"></script> 
  